@@ -333,6 +333,29 @@ class BufferedOutputBaseTest(unittest.TestCase):
             with smart_open.s3.open('thisbucketdoesntexist', 'mykey', 'wb') as fout:
                 fout.write(expected)
 
+    def test_read_nonexisting_key(self):
+        with self.assertRaises(ValueError):
+            with smart_open.s3.open(BUCKET_NAME, 'my_nonexisting_key', 'rb') as fin:
+                fin.read()
+
+    def test_double_close(self):
+        create_bucket_and_key()
+
+        text = u'там за туманами, вечными, пьяными'.encode('utf-8')
+        fout = smart_open.s3.open(BUCKET_NAME, 'key', 'wb')
+        fout.write(text)
+        fout.close()
+        fout.close()
+
+    def test_flush_close(self):
+        create_bucket_and_key()
+
+        text = u'там за туманами, вечными, пьяными'.encode('utf-8')
+        fout = smart_open.s3.open(BUCKET_NAME, 'key', 'wb')
+        fout.write(text)
+        fout.flush()
+        fout.close()
+
 
 class ClampTest(unittest.TestCase):
     def test(self):
@@ -373,7 +396,6 @@ class IterBucketTest(unittest.TestCase):
         expected = ['key_%d' % x for x in range(num_keys)]
         self.assertEqual(sorted(keys), sorted(expected))
 
-    @unittest.skip('this test takes too long for some unknown reason')
     def test_list_bucket_long(self):
         num_keys = 1010
         populate_bucket(num_keys=num_keys)
